'use strict';

var _interopRequireWildcard = function (obj) { return obj && obj.__esModule ? obj : { 'default': obj }; };

exports.__esModule = true;

var _alt$lazy$match$seq$str = require('parsly/parser');

var _ident$lex$sep = require('./util');

var _json = require('./json');

var _json2 = _interopRequireWildcard(_json);

// Punctuations
// ------------

var braceL = _ident$lex$sep.lex(_alt$lazy$match$seq$str.str('{'));
var braceR = _ident$lex$sep.lex(_alt$lazy$match$seq$str.str('}'));
var parenL = _ident$lex$sep.lex(_alt$lazy$match$seq$str.str('('));
var parenR = _ident$lex$sep.lex(_alt$lazy$match$seq$str.str(')'));
var angleL = _ident$lex$sep.lex(_alt$lazy$match$seq$str.str('<'));
var angleR = _ident$lex$sep.lex(_alt$lazy$match$seq$str.str('>'));
var period = _ident$lex$sep.lex(_alt$lazy$match$seq$str.str('.'));
var comma = _ident$lex$sep.lex(_alt$lazy$match$seq$str.str(','));
var amp = _ident$lex$sep.lex(_alt$lazy$match$seq$str.str('&'));

// Primitive types
// ---------------

var literal = _json2['default'].as('value');
var identifier = _ident$lex$sep.ident.as('id');
var variable = _alt$lazy$match$seq$str.seq(angleL, _ident$lex$sep.ident.as('variable'), angleR);
var uint = _alt$lazy$match$seq$str.match(/(?:0|[1-9]\d*)/);
var ref = _alt$lazy$match$seq$str.seq(amp, uint.as('ref'));

// Rules
// -----

var arg = _alt$lazy$match$seq$str.alt(variable, ref, literal);
var call = _alt$lazy$match$seq$str.seq(identifier, parenL, _ident$lex$sep.sep(arg, comma).as('args'), parenR);
var calls = _alt$lazy$match$seq$str.seq(period, call).repeat(1).as('calls');

var field = _alt$lazy$match$seq$str.lazy(function () {
  return _alt$lazy$match$seq$str.seq(identifier, calls.maybe(), fields.maybe());
});
var fields = _alt$lazy$match$seq$str.seq(braceL, _ident$lex$sep.sep(_alt$lazy$match$seq$str.alt(field, ref), comma), braceR).as('fields');

var query = _alt$lazy$match$seq$str.seq(call, fields);
var fragment = _alt$lazy$match$seq$str.seq(_ident$lex$sep.ident.as('type'), fields);
var node = _alt$lazy$match$seq$str.alt(query, fragment);

exports['default'] = node;
module.exports = exports['default'];