'use strict';

var _interopRequireWildcard = function (obj) { return obj && obj.__esModule ? obj : { 'default': obj }; };

exports.__esModule = true;

/**
 * Parses the given GraphQL query and returns its resulting AST.
 */
exports['default'] = parse;

var _filter = require('lodash/collection/filter');

var _filter2 = _interopRequireWildcard(_filter);

var _isPlainObject = require('lodash/lang/isPlainObject');

var _isPlainObject2 = _interopRequireWildcard(_isPlainObject);

var _Transformer$simple$subtree = require('parsly/transformer');

var _Transformer$simple$subtree2 = _interopRequireWildcard(_Transformer$simple$subtree);

var _graphql = require('./graphql');

var _graphql2 = _interopRequireWildcard(_graphql);

var _import = require('../types');

var t = _interopRequireWildcard(_import);

var filterOrNull = function filterOrNull(arr, fn) {
  arr = _filter2['default'](arr, fn);
  return arr.length > 0 ? arr : null;
};

var transformer = new _Transformer$simple$subtree2['default']()
// Root nodes
.rule({ id: _Transformer$simple$subtree.simple, args: _Transformer$simple$subtree.subtree, fields: _Transformer$simple$subtree.subtree }, function (_ref) {
  var id = _ref.id;
  var args = _ref.args;
  var fields = _ref.fields;
  return t.query(id, args, filterOrNull(fields, t.isField), filterOrNull(fields, t.isReference));
}).rule({ type: _Transformer$simple$subtree.simple, fields: _Transformer$simple$subtree.subtree }, function (_ref2) {
  var type = _ref2.type;
  var fields = _ref2.fields;
  return t.fragment(type, filterOrNull(fields, t.isField), filterOrNull(fields, t.isReference));
})
// Fields
.rule({ id: _Transformer$simple$subtree.simple, args: _Transformer$simple$subtree.subtree }, function (_ref3) {
  var id = _ref3.id;
  var args = _ref3.args;
  return t.call(id, args);
}).rule({ id: _Transformer$simple$subtree.simple }, function (_ref4) {
  var id = _ref4.id;
  var fields = _ref4.fields;
  var calls = _ref4.calls;
  return t.field(id, filterOrNull(fields, t.isField), filterOrNull(fields, t.isReference), calls);
})
// Arguments
.rule({ ref: _Transformer$simple$subtree.simple }, function (_ref5) {
  var ref = _ref5.ref;
  return t.reference(parseInt(ref, 10));
}).rule({ variable: _Transformer$simple$subtree.simple }, function (_ref6) {
  var variable = _ref6.variable;
  return t.identifier(variable);
}).rule({ value: _Transformer$simple$subtree.subtree }, function (_ref7) {
  var value = _ref7.value;
  return t.literal(JSON.parse(value), value);
})
// Error
.rule(_isPlainObject2['default'], function (node) {
  throw new Error('Unexpected node ' + JSON.stringify(node));
});
function parse(source) {
  var ast = _graphql2['default'].parse(String(source));
  return transformer.run(ast);
}

module.exports = exports['default'];