'use strict';

exports.__esModule = true;

var _custom$seq = require('parsly/parser');

function isWhitespace(ch) {
  return ch === 32 || ch >= 9 && ch <= 13;
}

function isIdentifierStart(ch) {
  return ch === 36 || ch === 95 || ch >= 65 && ch <= 90 || ch >= 97 && ch <= 122;
}

function isIdentifierChar(ch) {
  return ch === 36 || ch === 95 || ch >= 65 && ch <= 90 || ch >= 97 && ch <= 122 || ch >= 48 && ch <= 57;
}

/**
 * Matches whitespaces.
 */
var ws = _custom$seq.custom(function (resolve, reject) {
  return function (stream, index) {
    while (isWhitespace(stream.charCodeAt(index))) {
      index++;
    }

    return resolve(index);
  };
});

exports.ws = ws;
/**
 * Creates a new parser that skips whitespaces before parsing the token.
 */
var lex = function lex(atom) {
  return _custom$seq.custom(function (resolve, reject) {
    return function (stream, index) {
      var result = ws.run(stream, index);
      return atom.run(stream, result.index).aggregate(result);
    };
  });
};

exports.lex = lex;
/**
 * Matches identifiers.
 * The implementation does not handle astal range.
 */
var ident = lex(_custom$seq.custom(function (resolve, reject) {
  return function (stream, index) {
    if (isIdentifierStart(stream.charCodeAt(index))) {
      var start = index;
      while (isIdentifierChar(stream.charCodeAt(index))) {
        index++;
      }

      return resolve(index, stream.slice(start, index));
    }

    return reject(index, 'Expected valid identifier');
  };
}));

exports.ident = ident;
/**
 * Creates a parser that match `glue` separated tokens.
 */
var sep = function sep(atom, glue) {
  return _custom$seq.seq(_custom$seq.seq(atom, glue).many(), atom).repeat(0, 1);
};
exports.sep = sep;